package geotrellis.feature.op.geometry

import geotrellis._
import geotrellis.feature._
import com.vividsolutions.jts.{ geom => jts }

/**
 * Given a Geometry object, inspect the underlying geometry type
 * and recursively flatten it if it is a GeometryCollection
 */
case class FlattenGeometry[D](g1: Op[Geometry[D]]) extends Operation[List[Geometry[D]]] {

  def flattenGeometry(g: jts.Geometry): List[jts.Geometry] = g match {
    case g: jts.GeometryCollection => (0 until g.getNumGeometries).flatMap(
      i => flattenGeometry(g.getGeometryN(i))).toList
    case l: jts.LineString => List(l)
    case p: jts.Point => List(p)
    case p: jts.Polygon => List(p)
  }

  def _run(context: Context) = runAsync(List(g1))
  val nextSteps: Steps = {
    case a :: Nil => {
      val g = a.asInstanceOf[Geometry[D]]
      val geoms = flattenGeometry(g.geom)
      Result(geoms.map(geom => Feature(geom, g.data)))
    }
  }
}
